#include "Config.h"
#include "Defines.h"

Config::Config()
: m_bAllowMouse(true), m_nPlayerStanding(1000), m_nBlinkDuration(5)
{
   ArchiveCreate(&m_Archive);

   OpenArchiveFile(m_Archive, "DealOrNoDealSettings.tns");

   int nSettings = GetNumberArchives(m_Archive, "Settings");
   char strName[MAX_NAME_LENGTH];
   for(int i=0; i<nSettings; i++) {
      strcpy(strName, GetName(m_Archive, "Settings", i));

      if( strcmp(strName, "AllowMouse") == 0 ) {
         if( strcmp("0", GetValue(m_Archive, "Settings", i)) == 0 ) {
            m_bAllowMouse = false;
         } else {
            m_bAllowMouse = true;
         }
      }

      if( strcmp(strName, "Points") == 0 ) {
         m_nPlayerStanding = atoi(GetValue(m_Archive, "Settings", i));
      }

      if( strcmp(strName, "LabelBlinkDuration") == 0 ) {
         m_nBlinkDuration = atoi(GetValue(m_Archive, "Settings", i));
      }
   }
}

Config::~Config()
{
   ArchiveSetBatchMode(m_Archive, ARCHIVE_ENABLE_BATCH);
   UpdateArchiveEntry(m_Archive, "Settings", "AllowMouse", m_bAllowMouse ? "1" : "0", NULL);
   char buffer[10];
   Deal_itoa(m_nPlayerStanding, buffer, 10);
   UpdateArchiveEntry(m_Archive, "Settings", "Points", buffer, NULL);
   Deal_itoa(m_nBlinkDuration, buffer, 10);
   UpdateArchiveEntry(m_Archive, "Settings", "LabelBlinkDuration", buffer, NULL);
   ArchiveSetBatchMode(m_Archive, ARCHIVE_DISABLE_BATCH);

   ArchiveFree(&m_Archive);
}

bool Config::GetAllowMouse() const
{
   return m_bAllowMouse;
}

void Config::SetAllowMouse(bool bEnable)
{
   m_bAllowMouse = bEnable;
}

int Config::GetBlinkDuration() const
{
   return m_nBlinkDuration;
}

void Config::SetBlinkDuration(int nDuration)
{
   m_nBlinkDuration = nDuration;
}

int Config::GetPlayerStanding() const
{
   return m_nPlayerStanding;
}

void Config::ResetPlayerStanding()
{
   m_nPlayerStanding = 1000;
}

void Config::AdjustPlayerStanding(int nAmount)
{
   m_nPlayerStanding = Deal_Max(0, m_nPlayerStanding + nAmount);
}
